
/*
**      AUTOR       :  Leszek Chmielewski
**      NAZWA PLIKU :  lc_io.h
**
**      KOMENTARZ OGOLNY :  Procedury wejscia i wyjscia.
**                          Tylko dla trybow graficznych.
**
**  UWAGA:
**       Biblioteka ma dwie wersje: dla procesora 8086+emulator - z
**     litera E dolaczona do nazwy, i dla procesora 80286+7 - z cyfra 7,
**     tzn: LC_IOE.LIB; LC_IO7.LIB.
**
** Oprogramowanie powstalo w ramach projektu nr 8 8055 2 91, finansowanego
** w latach 1992-93 przez Komitet Badan Naukowych.
** Projekt zostal zrealizowany w Zespole Badan Stosowanych EPSILON Sp. z o.o.,
** ul. Danilowiczowska 11/66, tel. (22) 26 03 42
*/

#if !defined(LC_IO_INCLUDED)
  #define LC_IO_INCLUDED

#include <stdio.h>


#undef  OK
#define OK          0
/* wartosc nie odpowiadajaca zadnemu klawiszowi */
#define NEITHER_KEY -1
/* wartosci zwracane przez FileNameStatus() */
#define FN_BAD      1
#define FN_EXISTS   2


int FileNameStatus(const char *Fnam,const char *mod) ;
/*
  Sprawdza, czy plik o nazwie 'Fnam' nadaje sie do: czytania - jesli 'mod'
ma wartosc "r" (Read); pisania - jesli 'mod' ma wartosc "w" (Write).
  Dla 'mod' == "r" zwraca:
OK jesli plik istnieje;
FN_BAD jesli plik nie istnieje.
  Dla 'mod' == "w" zwraca:
OK jesli plik nie istnieje, a 'Fnam' jest prawidlowa nazwa pliku, tzn. nazwa
   pliku, ktory daje sie utworzyc;
FN_EXISTS jesli plik juz istnieje;
FN_BAD w pozostalych przypadkach.
  Funkcja pozostawia po sobie porzadek, tzn. nie zostawia zadnych wczesniej
nie istniejacych plikow, i nie kasuje wczesniej istniejacych.
  Jesli 'mod' ma wartosc inna niz "r" lub "w", to taki przypadek traktuje sie
jak blad fatalny: nastepuje przerwanie dzialania programu.
PRZYKLAD:
FILE *F ;
int _status ;
const char Fnam[] = "H:\binary\file.bin" ;
  if ((_status = FileNameStatus(Fnam,"w")) == OK)
    //teraz mozemy plik otworzyc do zapisu, nie sparawdzajac poprawnosci
    F = fopen(Fnam,"wb") ;
  else {
    switch(_status) {
    case FN_EXISTS:
      F = fopen(Fnam,"wb") ;
      printf("\nUWAGA: stara zawartosc pliku zostala skasowana !\n") ;
      break ;
    case FN_BAD:
      printf("\nBLAD: nie moge utworzyc pliku !\n") ;
      break ;
    } //switch
  } //if-else
*/


int  ReadLine(FILE* txtF,char *txt,int Uchr) ;
/*
  Czyta linie tekstu z pliku 'txtF' do tablicy znakowej 'txt', poczawszy od
aktualnie dostepnego znaku w pliku.
  Najwiekszym dopuszczalnym indeksem znaku w 'txt' jest 'Uchr'. Zatem, funkcja
czyta najwyzej 'Uchr' znakow [0..Uchr-1], pozostawiajac miejsce na koncowy '\0',
ktory dostawia, a reszte, jesli linia w pliku jest dluzsza, pomija, az do znaku
nowej lini wlacznie. Tablica 'txt' musi miec miejsce aby zmiescic 'Uchr+1'
znakow (+1 dla '\0'). Innymi slowy, jej deklaracja powinna byc rownowazna:
char txt[Uchr+1];
  Pierwszym znakiem dostepnym w pliku po wykonaniu funkcji jest pierwszy znak
w nowej linii, lub EOF, jesli lina czytana byla ostatnia w pliku 'txtF'.
  Plik 'txtF' musi byc uprzednio otwarty do czytania w trybie tekstowym.
  Zwraca liczbe znakow wprowadzonych do 'txt', nie liczac konczacego '\0'.
Jesli nie wczytano zadnego znaku, a napotkano koniec pliku, zwraca EOF.
*/


int ReadLineQuick(FILE *txtF,char *txt);
/*
  Does the same as ReadLine, but does not check for room in txt - this
makes it possible to read several times quicker.
The difference between this function and
return fscanf(txtF,"%[^\n]\n",txt);
is that fscanf omits the leading white signs, while ReadLineQuick() doesn't.
*/


int ReadLine2(FILE *inpF,char *txt,int Utxt,int comment_char);
/*
  Podobnie jak funkcja ReadLine(), z pliku 'inpF' czyta 1 linie i umieszcza ja
w tablicy 'txt'. Plik musi byc otwarty od odczytu. 'Utxt' jest najwiekszym
dopuszczalnym indeksem w 'txt'.
  ReadLine2() rozni sie od ReadLine1() tym, ze:
1. linie puste sa pomijane;
2. znaki nastepujace po znaku 'comment_char' sa pomijane wraz z tym znakiem;
   jest to znak komentarza;
3. znaki "biale" rozpoczynajace linie sa pomijane.
4. znaki "biale" konczace wczytany tekst sa pomijane.
Funkcja zwraca liczbe linii odczytanych z pliku danych. Jesli np. plik
zawieral linie (zawartosc linii - za strzalka '->'; comment_char='#'):
1->##
2->
3->  #comment
4->  AsRt--++7  # Komentarz
5->NOWA_LINIA
to funkcja umiesci w tablicy 'txt' tekst "AsRt7" i konczacy znak '\0', i
zwroci wartosc 4, bo aby uzyskac ten tekst odczytala 4 linie. Po wykonaniu
funkcji, nastepnym znakiem dostepnym w pliku bedzie 'N'.
  Jesli PRZED WCZYTANIEM niepustej linii funkcja napotka koniec pliku, lub
mial miejsce blad odczytu (np. plik nie byl otwarty do odczytu), funkcja
zwraca liczbe odczytanych linii z odwroconym znakiem, a tablica 'txt'
zawiera lancuch pusty "".
*/



void NextLine(FILE* txtF) ;
/*
  Wywolanie powoduje pominiecie znakow w otwartym pliku tekstowym 'txtF', az
do najblizszego znaku  nowej linii, lub konca pliku, jesli zostanie on
napotkany wczesniej.
  Po wykonaniu dostepny jest pierwszy znak nastepujacu po znaku nowej linii,
lub znak konca pliku, o ile zostal napotkany wczesniej. Napotkanie konca pliku
nie powoduje bledu.
PRZYKLAD:
/#
  Fragment czytanego pliku parametrow parF (PARameters File) wyglada jak
nastepuje. Po znaku # sa komentarze, ktorych nie chcemy czytac, ale nie
zamierzamy tez rozpoznawac komentarzy, a zadowalamy sie pomijaniem
wszystkego w wierszu, poza pierwsza liczba.

 12   # dana ipar1
 3.13
 7    # dane fpar2 i ipar3

#/
FILE  *parF ;
int   ipar1,ipar3 ;
float fpar2 ;
  fscanf(parF," %d",ipar1) ;  NextLine(parF) ;
  fscanf(parF," %f",fpar2) ;  NextLine(parF) ;
  fscanf(parF," %d",ipar3) ;  NextLine(parF) ;
*/



/* odczytywanie liczby podanego typu z lancucha, z pomoca funkcji sscanf() */
int StrToLong(const char *Pstr, long *Pout);
int StrToDouble(const char *Pstr, double *Pout);



int Yes(const char *msg);
int Tak(const char *msg);
/* Pytanie o tresci 'msg' w trybie tekstowym - wersja angielskojezyczna: Yes()
i polskojezyczna: Tak(). Ten opis dotyczy Yes(); dla Tak() jest odpowienio.
Nacisniecie 'y' lub  <CR> to odpowiedz "tak" - funkcja zwraca 1;
nacisniecie 'n' lub <ESC> to odpowiedz "nie" - funkcja zwraca 0.
Inne klawisze sa ignorowane.
  Funkcja sama dostawia za 'msg' tekst: " Y<CR> / N<ESC> ". Nie wysyla
zadnych znakow nowej linii.
  Przyklad:

  if (Yes("\nExit the program?)
    return;
  else
    continue;
*/

#endif  /* !defined(LC_IO_INCLUDED) */
