
#include <malloc.h>
#include "pic.h"


int CountPic(Tpic *Ppic,int mod,int Zx,int Zy,...);
int AssignPictureData(Tpic *Ppic,Tbrh *PpixA);



void InitPic(Tpic *Ppic) {
  Ppic->Pdat = NULL; /* this value signifies if picture is allocated */
} /* InitPic */



int CountPic(Tpic *Ppic,int mod,int Zx,int Zy,...) {
  int locUx;

  if (Zx<=0 || Zy<=0)
    return(BAD_DATA);

  Ppic->Zx=Zx; Ppic->Zy=Zy;
  Ppic->Zz = 0;
  Ppic->Zpix=sizeof(Tbrh);
  Ppic->mod=mod;

  switch (mod) {
  case BINpic:
    locUx = xBIT1pic(Zx-1);
    break;
  case BIT2pic:
    locUx = xBIT2pic(Zx-1);
    break;
  case BIT4pic:
    locUx = xBIT4pic(Zx-1);
    break;
  case GREpic:
    locUx = Zx-1;
    break;
  case CLRpic:
    Ppic->Zpix=sizeof(Tclr); locUx=Zx-1;
    break;
  default:
    return(BAD_DATA);
  } /* switch (mod) */

  Ppic->Zrow=(locUx+1)*Ppic->Zpix;
  Ppic->Zpic=Ppic->Zy*Ppic->Zrow;

  return OK;
} /* CountPic */



int AssignPictureData(Tpic *Ppic,Tbrh *PpixA) {
  int y;

  if (PpixA == NULL)
    return NO_MEMORY;

  if ((Ppic->Pdat = (char*)malloc(Ppic->Zy*sizeof(char*))) == NULL) {
    FreePic(Ppic); return NO_MEMORY;
  }

  for (y = 0; y < Ppic->Zy; y++)
    ((Tbrh**)(Ppic->Pdat))[y] = &((PpixA)[Ppic->Zrow*y]);

  return OK;
} /* AssignPictureData */



int AllocPic(Tpic *Ppic,int mod,int Zx,int Zy,...) {
  Tbrh *PpixA;
  int result;

  FreePic(Ppic);

  if ((result = CountPic(Ppic,mod,Zx,Zy)) != OK)
    return result;

  if ((PpixA=(Tbrh*)malloc(Ppic->Zpic*sizeof(char))) == NULL) {
    FreePic(Ppic); return NO_MEMORY;
  }

  Ppic->isAlloc = 1;
  result = AssignPictureData(Ppic,PpixA);

  return result;
} /* AllocPic */



int AssignPic(Tpic *Ppic,Tbrh *PpixA,int mod,int Zx,int Zy,...) {
  int result;

  FreePic(Ppic);

  if ((result=CountPic(Ppic,mod,Zx,Zy)) != OK)
    return result;

  Ppic->isAlloc = 0;
  result = AssignPictureData(Ppic,PpixA);

  return result;
} /* AssignPic */



void FreePic(Tpic *Ppic) {
  if (Ppic->Pdat != NULL) {
    if (Ppic->isAlloc)
      Delete(_PpixA(Ppic));
    Delete(Ppic->Pdat);
    InitPic(Ppic);
  }
} /* FreePic */
