
#include <stdio.h>
#include <malloc.h>
#include "auxmacr.h"
#include "pic.h"

#include "skapicmk.h"

#define    BRH(Ppic,R,C)         GRE(Ppic,C,R)
#define SetBRH(Ppic,R,C,val)  SetGRE(Ppic,C,R,val)

/* typy i zmienne globalne */
/*
Udzialy liniowe piksli inp w pikslu out wzdluz jednej wspolrzednej: piksel
out pokrywa shrsta piksla Ista, cale piksle od Ista+1 do Iend-1, oraz shrend
piksla Iend. Jesli Iend<0 (=-1), to tylko caly Ista (wtedy shrxxx niewazne).
Udzialy shrxxx to liczby ulamkowe [0..1), ale przesuniete bitowo <<8.
Zatem np. 0.0 to 0x00, 0.5 to 0x80 (128/256), ~0.99 to 0xFD (253/256).
*/
/* Dynamiczne tablice udzialow dla R i C */
static Tshr *Rshr, *Cshr;

/* globalne udzialy 1 pix inp w 1 pix out, w kierunkach R i C, skalowane */
static unsigned long glbshrR,glbshrC;

/* wskaznik do obrazu wejsciowego (do komunikacji z Cloop()) */
static const Tpic *glbPinppic;

/* skalowanie powrotne; wynik (i dana) typu unsigned long */
#define rska(x)   ( ((x)+0x7fL)   >>  8 )
#define rska2(x)  ( ((x)+0x7fffL) >> 16 )
/* przeliczanie liczby typu unsigned long na byte, z kontrola przepelnienia */
#define l2b(x)    ( ((x) > 255) ? ((byte)255) : ((byte)(x)) )

static unsigned long Cloop(int R,int C2);



int ResizePic(const Tpic *Pinppic,Tpic *Poutpic, /* obrazy */
              const Twin *Pinpwin,const Twin *Poutwin /* okna */,const int mod) {
  register int  C2,R;
  int           R2,winZR,winZC;
  unsigned long outbrh;

  /* UWAGA: Ta wersja nie realokuje pamieci, jesli Poutpic jest za maly */
  /* kontrola danych */
  if ((_mod(Pinppic) != GREpic) || (_mod(Poutpic) != GREpic) ||
      !IsPicOK(Pinppic) || !IsWinOK(Pinpwin) || !IsWinInPic(Pinpwin,Pinppic) ||
      !IsPicOK(Poutpic) || !IsWinOK(Poutwin) || !IsWinInPic(Poutwin,Poutpic) ||
      !((mod == NORM) || (mod == QUICK) || (mod == MARK)) )
    return(BAD_DATA);

  if (mod == MARK)
    return ResizePicMark(Pinppic,Poutpic,Pinpwin,Poutwin);

  winZR = Poutwin->Zy;
  winZC = Poutwin->Zx;
  if ((Rshr = (Tshr *)malloc((winZR) * sizeof(Tshr))) == NULL)
    return(NO_MEMORY);
  if ((Cshr = (Tshr *)malloc((winZC) * sizeof(Tshr))) == NULL) {
    free(Rshr);
    return(NO_MEMORY);
  }

  glbPinppic = Pinppic;

  /* globalne udzialy jednego piksla wyjsciowego w wejsciowym */
  glbshrR = Find1Share(Pinpwin->My,_Xy(Pinpwin),Poutwin->My,_Xy(Poutwin));
  glbshrC = Find1Share(Pinpwin->Mx,_Xx(Pinpwin),Poutwin->Mx,_Xx(Poutwin));
  /* udzialy we wszystkich pikslach w kierunkach: R i C */
  FindShares(Rshr,_My(Pinpwin),_Xy(Pinpwin),_My(Poutwin),_Xy(Poutwin),mod);
  FindShares(Cshr,_Mx(Pinpwin),_Xx(Pinpwin),_Mx(Poutwin),_Xx(Poutwin),mod);

  for (R2=0; R2<winZR; R2++)
  for (C2=0; C2<winZC; C2++) {
    outbrh = 0L;
    if (Rshr[R2].Iend < 0) {
      /* tylko jeden piksel */
      outbrh = Cloop(Rshr[R2].Ista,C2);
    } else {
      /* posrednie piksle */
      for (R=Rshr[R2].Ista+1; R<Rshr[R2].Iend; R++)
        outbrh += Cloop(R,C2);
      outbrh <<= 8;  /* mnozenie przez przeskalowane 1.0 */
      /* pierwszy i ostatni piksel */
      outbrh += Rshr[R2].shrsta * Cloop(Rshr[R2].Ista,C2);
      outbrh += Rshr[R2].shrend * Cloop(Rshr[R2].Iend,C2);
      /* wazymy glbshrR i dwukrotnie reskalujemy: */
      /* ze wzgledu na wazenie przez shrxxx oraz glbshrR */
      outbrh *= glbshrR;
      outbrh = rska2(outbrh);
    } /* if-else(Rshr[R2].Iend < 0) */
    SetBRH(Poutpic,R2+Poutwin->My,C2+Poutwin->Mx,l2b(outbrh));
  } /* for (R2,C2) */

  free(Rshr);
  free(Cshr);
  return(OK) ;

} /* ResizePic */



unsigned long Cloop(int R,int C2) {
/* wykonuje petle kolumn, przy zadanym R, wedlug udzialow dla C2 */
/* zwraca otrzymana wazona sume jasnosci */
  unsigned long outbrh;
  register int  C;

  if (Cshr[C2].Iend < 0) {
    /* tylko jeden piksel */
    return((unsigned long)BRH(glbPinppic,R,Cshr[C2].Ista));
  }  
  /* posrednie piksle */
  outbrh = 0;
  for (C=Cshr[C2].Ista+1; C<Cshr[C2].Iend; C++)
    outbrh += (unsigned long)BRH(glbPinppic,R,C);
  outbrh <<= 8;
  /* pierwszy i ostatni piksel */
  outbrh += Cshr[C2].shrsta * (unsigned long)BRH(glbPinppic,R,Cshr[C2].Ista);
  outbrh += Cshr[C2].shrend * (unsigned long)BRH(glbPinppic,R,Cshr[C2].Iend);
  /* wazymy glbshrC i dwukrotnie reskalujemy: */
  /* ze wzgledu na wazenie przez shrxxx oraz glbshrC */
  outbrh *= glbshrC;
  outbrh = rska2(outbrh);

  return(outbrh);

} /* Cloop */



/* wersja z danymi w postaci liczb (int) */
int ResizePicInt(const Tpic *Pinppic,Tpic *Poutpic, /* obrazy */
  const int Mxinp,const int Myinp,const int Zxinp,const int Zyinp, /* okna */
  const int Mxout,const int Myout,const int Zxout,const int Zyout,
  const int mod) {

  Twin inpwin,outwin;

  inpwin.Mx = Mxinp;
  inpwin.My = Myinp;
  inpwin.Zx = Zxinp;
  inpwin.Zy = Zyinp;
  outwin.Mx = Mxout;
  outwin.My = Myout;
  outwin.Zx = Zxout;
  outwin.Zy = Zyout;

  return(ResizePic(Pinppic,Poutpic,&inpwin,&outwin,mod));

} /* ResizePicInt */



/* wersja z oknami zawierajacymi cale obrazki */
int ResizePicShort(const Tpic *Pinppic,Tpic *Poutpic,const int mod)
{
  Twin inpwin,outwin;
  
  Tpic2Twin(Pinppic,&inpwin);
  Tpic2Twin(Poutpic,&outwin);

  return(ResizePic(Pinppic,Poutpic,&inpwin,&outwin,mod));

} /* ResizePicShort */
