
#include <limits.h>
#include <math.h>
#include "auxmacr.h"
#include "pic.h"

/* tylko dla printf() - testy */
#include <stdio.h>


/* obiekty globalne dla modulu */
static int     Mi,Xi,Mo,Xo;
static Tauxsca i_fac,i_add;
static int     i_shf;
static double  glbfac;
/* mala liczba - "limit bezpieczenstwa" dla Min i maX */
static double  dlt;


/* funkcja skalujaca */
#define SCAL(I)  (int)( ((I)*i_fac + i_add) >> i_shf )

/* sparawdzanie, czy warunki skalowania sa spelnione */
#define OKsca    ((SCAL(Mi) == Mo) && (SCAL(Xi) == Xo))

/* zaokraglanie z (double) do (Tauxsca) */
#define scaround(x)  ( (Tauxsca)floor((x) + 0.5) )

/* wyliczanie (calkowitych) i_fac i i_add */
static void FindFA(void) {
  i_fac = scaround((Xo+1-Mo-2.0*dlt)*glbfac);
  i_add = scaround((Xi*(Mo+dlt)-Mi*(Xo+1.0-dlt))*glbfac);
} /* FindFA */



int PrepRngSca3(int Minp,int Xinp,int Mout,int Xout,
  Tauxsca *Pfac,Tauxsca *Padd,int *Pshf) {
/*
  Algorytm polega na tym, ze zmniejszamy "limity bezpieczenstwa":
'dlt' dla minimum i maksimum, tak, jak to tylko jest mozliwe,
sprawdzajac, czy warunki, jakie musi spelnic przeksztalcenie skalujace:
out = SCAL(inp) = ( inp * (*Pfac) + (*Padd) ) >> (*Pshf)
sa spelnione, tzn. czy (makro OKsca):
(SCAL(Minp) >= Mout) && (SCAL(Xinp) <= Xinp)
Konczymy, gdy dalsze zmniejszanie limitow jest niemozliwe, tzn. gdy ich
przyrosty sa bardzo male (<lim). Przyrosty sa zmniejszane przez bisekcje.
*/

  double d_fac,d_add,sca;
  double inc,lim;  /* przyrost 'dlt' i ich wartosc limitujaca */

  Mi = Minp;  Xi = Xinp;  Mo = Mout;  Xo = Xout;

  /* kontrola danych */
  if ((Mo > Xo) || (Mi > Xi))
    return(BAD_DATA);

  /* przypadki trywialne */
  if (Xo==Mo) {
    *Pfac = (Tauxsca)0;
    *Padd = (Tauxsca)Xo;
    *Pshf = 0;
    return(OK);
  }
  if ((Mi==Mo) && (Xi==Xo)) {
    *Pfac = (Tauxsca)1;
    *Padd = (Tauxsca)0;
    *Pshf = 0;
    return(OK);
  }
  if (Mi==Xi) {
    *Pfac = (Tauxsca)0;
    *Padd = scaround(0.5*(Mo+Xo));
    *Pshf = 0;
    return(COORDINATE_DEGENERATE);
  }

  /* szacowanie maksymalnego bezpiecznego czynnika skalujacego */
  d_fac = (double)(Xo+1.0-Mo)/(Xi-Mi);
  d_add = ((double)Xi*Mo-(Xo+1.0)*Mi)/(Xi-Mi);
  /* wartosci limitujace w roznych przypadkach, zapisane w 'dlt' i 'inc' */
  /* dodatkowe zabezpieczenie: nieco zwiekszone 'Xi' i zmniejszone 'd_add' */
  dlt = Uauxsca / ((Xi+2.0) * d_fac);
  if (d_add > 0.0)
    inc = Uauxsca / ((Xi+2.0) * d_fac + d_add);
  else if (d_add < 0.0)
    inc = fabs(Wauxsca / (d_add+2.0));
  else
    inc = Uauxsca;  /* bardzo duza liczba */
  /* minimum minimorum */
  dlt = min(dlt,inc);
  /* 'i_shf' to wykladnik potegi 2, dajacy nie wiecej niz 'dlt' */
  i_shf  = (int)floor(log(dlt)/log(2.0));
  sca    = (double)scaround(pow(2.0,(float)i_shf));
  glbfac = sca / (double)(Xi-Mi);

  /*
  Bardzo maly przyrost dlt, to znaczy taki przyrost, ktory nie moze
  zmienic wartosci (calkowtych!) i_fac, i_add, czyli zmienia je o mniej niz 1.
  Warunek dla i_add jest silniejszy, szczegolnie ze wzgledu na skladnik Xi*dlt
  (zob. wyr. na i_add w fukcji FindFA()). Dla pewnosci wartosc limitujaca
  przyjmujemy jeszcze znacznie mniejsza (czynnik na poczatku wyrazenia).
  */
  lim = 0.01 * (Xi-Mi) / (Xi*sca*2);

  /* szukanie wartosci poczatkowych zapewniajacych spelnienie warunku */
  inc = 0.1;  /* niewielki przyrost */
  dlt  = -2.0*inc;  /* na pewno zbyt male wartosci poczatkowe */
  do {
    if (dlt > 1.1) {  /* przyrosty byly za duze - od nowa */
      inc *= 0.5;
      if (inc < lim) {
        /* generalne fiasko */
        printf("\n\nGENERAL ERROR in PrepRngSca3()!\n");
        return(ERROR);
      }
      dlt  = -2.0*inc;
    }
    dlt += inc;
    FindFA();
  } while (!OKsca);
  /* glowny algorytm: szukanie minimalnej wartosci 'dlt' */
  inc = 1.0;  /* b.duze przyrosty poczatkowe */
  do {
    while (FindFA(), OKsca)
      dlt -= inc;
    dlt += inc;  /* powrot do ostatniej dobrej wartosci */
    inc *= 0.5;  /* bisekcja przyrostu */
  } while (inc >= lim);
  FindFA();

  /* ostateczny test wyniku */
  if (
  (rngsca3(Minp  ,i_fac,i_add,i_shf) != Mout) ||
  (rngsca3(Minp-1,i_fac,i_add,i_shf) >= Mout) ||
  (rngsca3(Xinp  ,i_fac,i_add,i_shf) != Xout) ||
  (rngsca3(Xinp+1,i_fac,i_add,i_shf) <= Xout)
  ) {
    printf("\n!ERROR in PrepRngSca3(%d,%d -> %d,%d;",
      Minp,Xinp,Mout,Xout);
    printf("f=%.0f a=%.0f s=%.0f);\n",
      (double)i_fac,(double)i_add,(double)i_shf);
    printf("Minp=%d Mout=%d (? != %d=skal(%d); %d=skal(%d))\n",
      Minp,Mout,
      rngsca3(Minp  ,i_fac,i_add,i_shf),Minp,
      rngsca3(Minp-1,i_fac,i_add,i_shf),Minp-1
    );
    printf("Xinp=%d Xout=%d (? != %d=skal(%d); %d=skal(%d))\n",
      Xinp,Xout,
      rngsca3(Xinp  ,i_fac,i_add,i_shf),Xinp,
      rngsca3(Xinp+1,i_fac,i_add,i_shf),Xinp+1
    );
    return(ERROR);
  }

  /* wartosci zwracane przez parametry */
  *Pfac = i_fac;
  *Padd = i_add;
  *Pshf = i_shf;
  return(OK);

} /* PrepRngSca3 */


int PrepRngSca(int Minp,int Xinp,int Mout,int Xout,
  Trngscainf *Pinf) {

  return(PrepRngSca3(Minp,Xinp,Mout,Xout,
    &(Pinf->fac),&(Pinf->add),&(Pinf->shf)));

} /* PrepRngSca */



int PrepCrdSca(int Minp,int Xinp,int Mout,int Xout,
  Tcrdscainf *Pinf) {

  int errM,errX;
  int Zout_inp = (Xout-Mout+1)/(Xinp-Minp+1) - 1;

  Zout_inp = max(Zout_inp,0);

  /* minima: kresy dolne zakresu */
  errM = PrepRngSca(Minp,Xinp,Mout,Xout - Zout_inp,&(Pinf->M));
  /* maksima - kresy gorne zakresu */
  errX = PrepRngSca(Minp,Xinp,Mout + Zout_inp,Xout,&(Pinf->X));

  if (errM!=OK || errX!=OK)
    return(BAD_DATA);
  else
    return(OK);

} /* PrepCrdSca */



int PrepWinSca(const Twin *Pinpwin,const Twin *Poutwin,
  Twinscainf *Psca) {

  int err_x,err_y;

  if (!IsWinOK(Pinpwin) || !IsWinOK(Poutwin))
    return(BAD_DATA);

  err_x = PrepCrdSca(Pinpwin->Mx,_Xx(Pinpwin),Poutwin->Mx,_Xx(Poutwin),
    &(Psca->x));
  err_y = PrepCrdSca(Pinpwin->My,_Xy(Pinpwin),Poutwin->My,_Xy(Poutwin),
    &(Psca->y));

  if (err_x!=OK || err_y!=OK)
    return(BAD_DATA);
  else
    return(OK);

} /* PrepWinSca */
