/*
** AUXILIARY, PRIVATE DEFINITINS AND MACROS FOR PICTURES
**
** Library name:                 pic
** This header file name:        picaux.h
** Other necessary libraries:    none
** Other necessary header files: this file to be used ONLY as included
**                               from pic.h
*/

#if !defined(Delete)
  /*Frees allocated memory block using the safe method*/
  #define Delete(p) if ((p)!=NULL) { free(p); p=NULL; }
#endif

/* pointer to the array of pixel values - for all except volume pictures! */
/* returned value is of type (Tbrh*) */
#define _PpixA(Ppic)  ( ((Tbrh**)((Ppic)->Pdat))[0] )
/* pointer to the array of pixel values - for volume pictures */
#define _PpixAv(Ppic) ( ((Tbrh***)((Ppic)->Pdat))[0][0] )
/* byte at (x,y), assuming a GREpic, which is the most simple case */
#define picBYT(Ppic,x,y)  ( ((Tbrh**)((Ppic)->Pdat))[y][x] )
/* pointer to byte at (x,y), assuming a GREpic */
#define PpicBYT(Ppic,x,y)  ( &picBYT(Ppic,x,y) )


/* auxiliary macros for counting columns in Nbit pictures */
/* byte # (x=column) in a row, in which the sought bit(s) corresponding to
   column x of the original array */
#define xBIT1pic(x)  ( (x) >> 3 )
#define xBIT2pic(x)  ( (x) >> 2 )
#define xBIT4pic(x)  ( (x) >> 1 )
/* bit #, in byte CNbitpic(C), corresponding to column x in orig.array -
   it is 1 bit for BIN, and the younger bit from 2 for BIT2 or 4 for BIT4 */
#define posBIT1pic(x)  ( (x)%8 )
#define posBIT2pic(x) ( ((x)%4) << 1 )
#define posBIT4pic(x) ( ((x)%2) << 2 )

/* auxiliary macros for setting and reading 1, 2 or 4 bits from position n
   in one byte B (n denotes the position of a younger one of two bits) */
typedef unsigned char         picbyt; /* a "picture" byte */
#define on_BIT(B,n)           ( (picbyt)((B) |  (0x0001<<(n))) )
#define offBIT(B,n)           ( (picbyt)((B) & ~(0x0001<<(n))) )
#define on_BIT1picBYT(B,n)    (B) = on_BIT(B,n)
#define offBIT1picBYT(B,n)    (B) = offBIT(B,n)
#define offBIT2picBYT(B,n)    (B) = (picbyt)((B) & ~(0x0003<<(n)))
#define offBIT4picBYT(B,n)    (B) = (picbyt)((B) & ~(0x000F<<(n)))
#define setBIT1picBYT(B,n,v)  (B) = (v)?(on_BIT(B,n)):(offBIT(B,n))
#define setBIT2picBYT(B,n,v)  { offBIT2picBYT(B,n); \
                                 (B) = (picbyt)((B) + ((v)<<(n))); }
#define setBIT4picBYT(B,n,v)  { offBIT4picBYT(B,n); \
                                 (B) = (picbyt)((B) + ((v)<<(n))); }

/* get... : r-values */
#define Zint   (8*sizeof(int))  /* to have it compiler independent */
#define getBIT1picBYT(B,n)  ( (unsigned)((B) << (Zint-1-(n))) >> (Zint-1) )
#define getBIT2picBYT(B,n)  ( (unsigned)((B) << (Zint-2-(n))) >> (Zint-2) )
#define getBIT4picBYT(B,n)  ( (unsigned)((B) << (Zint-4-(n))) >> (Zint-4) )

/* auxiliary functions */
void SetWinBIN(Tpic *Ppic,const Twin *Pwin,int bin);
void SetWinBIT2(Tpic *Ppic,const Twin *Pwin,int bit2);
void SetWinBIT4(Tpic *Ppic,const Twin *Pwin,int bit4);
void SetWinGRE(Tpic *Ppic,const Twin *Pwin,int brh);
void SetWinCLR(Tpic *Ppic,const Twin *Pwin,Tclr clr);
