/*
** DIAGNOSTICS OF PICTURES AND WINDOWS
**
** Library name:                 pic
** This header file name:        picdiag.h
** Other necessary libraries:    none
** Other necessary header files: this file to be used ONLY as included
**                               from pic.h
*/

#if !defined(PICDIAG_INCLUDED)
  #define PICDIAG_INCLUDED


/* info about a pic */

/* picture is allocated - not empty */
#define IsPicOK(Pp) ( (Pp)->Pdat != NULL )
/* fields in the Tpic structure */
#define _mod(Pp)  ( (Pp)->mod )
#define _Zx(Pp)  ( (Pp)->Zx )
#define _Zy(Pp)  ( (Pp)->Zy )
#define _Zz(Pp)  ( (Pp)->Zz )
#define _Ux(Pp)  ( (Pp)->Zx - 1 )
#define _Uy(Pp)  ( (Pp)->Zy - 1 )
#define _Uz(Pp)  ( (Pp)->Zz - 1 )
#define SIZEOFpicP(Pp) ( (Pp)->Zpic )

/* Contiguous memory needed for pixels (not for intermediate data!) of a
** picture of size Zx columns and Zy rows */
#define SIZEOFgrepic(Zx,Zy)  ( (Zy)*(Zx) )
#define SIZEOFclrpic(Zx,Zy)  ( (Zy)*(Zx)*3 )
#define SIZEOFbinpic(Zx,Zy)  ( (Zy)*xBIT1pic(Zx) )
#define SIZEOFbit2pic(Zx,Zy) ( (Zy)*xBIT2pic(Zx) )
#define SIZEOFbit4pic(Zx,Zy) ( (Zy)*xBIT4pic(Zx) )
#define SIZEOFpic(mod,Zx,Zy) ( (mod == BINpic) ?  \
                               SIZEOFbinpic(Zx,Zx) :  \
                             (mod == BIT2pic) ?  \
                               SIZEOFbit2pic(Zx,Zy) :  \
                             (mod == BIT4pic) ?  \
                               SIZEOFbit4pic(Zx,Zy) :  \
                             (mod == GREpic) ?  \
                               SIZEOFgrepic(Zx,Zy) :  \
                             (mod == CLRpic) ?  \
                               SIZEOFclrpic(Zx,Zy) :  \
                             ( -1 ) )


/* Predicates on pictures and windows; pointers to arguments are used */

#define PicsEqual(Pp1,Pp2) ( ((Pp1)->Zx == (Pp2)->Zx) &&  \
                             ((Pp1)->Zy == (Pp2)->Zy) )
#define VPicsEqual(Pp1,Pp2) ( ((Pp1)->Zx == (Pp2)->Zx) &&  \
                              ((Pp1)->Zy == (Pp2)->Zy) &&  \
                              ((Pp1)->Zz == (Pp2)->Zz) )

/* A "good" window contains at least 1 pixel and has nonnegative coordinates */
#define IsWinOK(Pw)   ( ((Pw)->Mx >= 0) && ((Pw)->Zx > 0) && \
                        ((Pw)->My >= 0) && ((Pw)->Zy > 0) )
#define IsVWinOK(Pw)  ( ((Pw)->Mx >= 0) && ((Pw)->Zx > 0) && \
                        ((Pw)->My >= 0) && ((Pw)->Zy > 0) && \
                        ((Pw)->Mz >= 0) && ((Pw)->Zz > 0) )

/* windows are identical; "goodness" not checked */
#define WinsEqual(Pw1,Pw2)  ( ((Pw1)->Mx == (Pw2)->Mx) &&  \
                              ((Pw1)->My == (Pw2)->My) &&  \
                              ((Pw1)->Zx == (Pw2)->Zx) &&  \
                              ((Pw1)->Zy == (Pw2)->Zy) )
#define VWinsEqual(Pw1,Pw2) ( ((Pw1)->Mx == (Pw2)->Mx) &&  \
                              ((Pw1)->My == (Pw2)->My) &&  \
                              ((Pw1)->Mz == (Pw2)->My) &&  \
                              ((Pw1)->Zx == (Pw2)->Zx) &&  \
                              ((Pw1)->Zy == (Pw2)->Zy) &&  \
                              ((Pw1)->Zz == (Pw2)->Zz) )

/* Window 2 is contained in window 1; "goodness" not checked */
#define  IsWin1In2(Pw1,Pw2) \
  ( ((Pw1)->Mx>=(Pw2)->Mx) && (_Xx(Pw1)<=_Xx(Pw2)) && \
    ((Pw1)->My>=(Pw2)->My) && (_Xy(Pw1)<=_Xy(Pw2)) )
#define  IsVWin1In2(Pw1,Pw2) \
  ( ((Pw1)->Mx>=(Pw2)->Mx) && (_Xx(Pw1)<=_Xx(Pw2)) && \
    ((Pw1)->My>=(Pw2)->My) && (_Xy(Pw1)<=_Xy(Pw2)) && \
    ((Pw1)->Mz>=(Pw2)->Mz) && (_Xz(Pw1)<=_Xz(Pw2)) )

/* Window *Pw is contained in picture *Pp; "goodness" not checked */
#define IsWinInPic(Pw,Pp)  ( ((Pw)->Mx >= 0) && (_Xx(Pw) <= (Pp)->Zx-1) &&  \
                             ((Pw)->My >= 0) && (_Xy(Pw) <= (Pp)->Zy-1) )
#define IsVWinInPic(Pw,Pp) ( ((Pw)->Mx >= 0) && (_Xx(Pw) <= (Pp)->Zx-1) &&  \
                             ((Pw)->My >= 0) && (_Xy(Pw) <= (Pp)->Zy-1) &&  \
                             ((Pw)->Mz >= 0) && (_Xz(Pw) <= (Pp)->Zz-1) )

/* Pixel (x,y) is in picture *Pp; "goodness" not checked */
#define IsPixInPic(x,y,Pp)  ( ((x) >=        0) &&  \
                              ((x) <  (Pp)->Zx) &&  \
                              ((y) >=        0) &&  \
                              ((y) <  (Pp)->Zy) )
#define IsVPixInPic(x,y,Pp) ( ((x) >=        0) &&  \
                              ((x) <  (Pp)->Zx) &&  \
                              ((y) >=        0) &&  \
                              ((y) <  (Pp)->Zy) &&  \
                              ((z) >=        0) &&  \
                              ((z) <  (Pp)->Zz) )

/* Pixel (x,y) is in window *Pw; "goodness" not checked */
#define IsPixInWin(R,C,Pw)    ( ((x) >= (Pw)->Mx) &&  \
                                ((x) <=  _Xx(Pw)) &&  \
                                ((y) >= (Pw)->My) &&  \
                                ((y) <=  _Xy(Pw)) )
#define IsVPixInVWin(R,C,Pw)  ( ((x) >= (Pw)->Mx) &&  \
                                ((x) <=  _Xx(Pw)) &&  \
                                ((y) >= (Pw)->My) &&  \
                                ((y) <=  _Xy(Pw)) &&  \
                                ((z) >= (Pw)->Mz) &&  \
                                ((z) <=  _Xz(Pw)) )


#endif /* !defined(PICDIAG_INCLUDED) */
