/*
** SCALING OF THE PICTURES AND WINDOWS
**
** Library name:                 pic
** This header file name:        picscal.h
** Other necessary libraries:    none
** Other necessary header files: this file to be used ONLY as included
**                               from pic.h
*/

#if !defined(PICSCAL_INCLUDED)
  #define PICSCAL_INCLUDED

/* modes of operation for functions ResizePic(), FindShares() */
/* with grey-level iterpolation: */
#define NORM  1
/* with line cancelling/multiplying, depending on scale: */
#define QUICK 2
/* any non-zero pixel in input will be marked in output with the same grey */
/* level, even if image is reduced; lower lines can cover upper lines: */
#define MARK  3


/* SCALING (RESIZING) THE WINDOWS */

int ProportionWin1As2(Twin *Pwin1,const Twin *Pwin2);
/*
  Zmniejsza okno 1 tak, aby jego proporcje staly sie takie same, jak proporcje
okna 2. Zmniejszana jest jedna wspolrzedna prawego dolnego rogu: XR lub XC,
ta, w ktorej kierunku okno 1 jest "mniej smukle" niz 2. X? jest zaokraglane.
Gwarantuje sie, ze *Pwin1 nie zostanie powiekszone.
PRZYKLADY:  ( okno: {Mx,My,Xx,Xy} )
  1. w1: {0,0,3,8} /# okno  4:7  = 1.00:1.75; zmniejszyc 7 #/
     w2: {0,0,7,7} /# okno  8:8  = 1.00:1.00 #/
     --->  w1: {0,0,3,3} /# okno 4:4 = 1.00:1.00, bo 4*8/8 = 4 #/
  2. w1: { 1, 1, 5, 4} /# okno  5:4  = 1.25:1.00 = 1.00:0.80; zmniejszyc 5 #/
     w2: {11,11,21,28} /# okno 11:18 = 0.61:1.00 = 1.00:1.63 #/
     --->  w1: {1,1,2,4} /# okno 2:4 = 0.50:1.00, bo 4*11/18 = 2.44 -> 2 #/
Zwraca: OK, BLAD (okno(a) nie OK).
*/

int TruncateWin1Into2(Twin *Pw1,const Twin *Pw2);
/*
  If necessary, clips window 1 so as it is contained in window 2.
Returns: OK or ERROR if any window is not "good" or window 1 has
no common pixels with window 2. If ERROR, window 1 is not changed.
*/

int RescaleWin1From2To3(Twin *Pw1,const Twin *Pw2,const Twin *Pw3);
/*
  Changes window *Pw1 so as it had been formed by rescaling the given window
*Pw1 from window *Pw3 into window *Pw4. The new *Pw1 is in such a relation to
the old *Pw1, as *Pw2 to *Pw3. Proportions of *Pw1 change accordingly.
  *Pw1 must be contained in *Pw2. This is checked, while the "goodness" of
the windows is not checked.
  For how the scaling is performed see functions Prep???Sca() i ???sca().

Returned values: OK, ERROR.
*/

int CorrectWin2Pic(Twin *Pwin,int picZx,int picZy);
/*
  If necessary, window *Pwin is corrected so that it is inside the picture
of dimensions picZx, picZy. The resulting window is always "good", although
if it was outside the picture it will become a 1-pixel window.
  Returns 1 if there was a correction and 0 otherwise.
*/


/* SCALING (RESIZING) THE IMAGES */

/* wersja skrocona operujaca na calych obrazach */
int ResizePicShort(const Tpic *Pinppic,Tpic *Poutpic,const int mod);
/* wersja z danymi w postaci okien (Twin) */
int ResizePic(const Tpic *Pinppic,Tpic *Poutpic,
              const Twin *Pinpwin,const Twin *Poutwin,const int mod);
/* wersja z danymi w postaci liczb (int) */
int ResizePicInt(const Tpic *Pinppic,Tpic *Poutpic, /* obrazy */
  const int Mxinp,const int Myinp,const int Zxinp,const int Zyinp, /* okna */
  const int Mxout,const int Myout,const int Zxout,const int Zyout,
  const int mod);
/*
**     OPIS:
**       Funkcja realizuje afiniczne przeskalowanie prostokatnego
**     okna obrazu *Pinppic o wymiarach MRinp, ..., XCinp w prostokatne
**     okno obrazu *Poutpic o wymiarach MRout, ..., XCout.
**       Jesli 'mod==NORM', to jasnosc piksla wyjsciowego rowna sie sredniej
**     jasnosci piksli wejsciowych, pokrywanych przez ten piksel, wazonej
**     ich udzialami powierzchni w pikslu wyjsciowym.
**       Jesli 'mod==QUICK', to jasnosc piksla wyjsciowego rowna sie jasnosci
**     tego piksla wejsciowego, w ktorego polu lezy srodek piksla wyjsciowego.
**       Jesli 'mod==MARK', to kazdemu pikslowi obrazu wejsciowego bedzie
**     odpowiadal PRZYNAJMNIEJ JEDEN piksel obrazu wyjsciowego, nawet, jesli
**     obraz wyjsciowy jest mniejszy niz wejsciowy. Ten tryb sluzy do
**     zaznaczania (MARKing) na obrazie wyjsciowym tych piksli, ktore sa
**     niezerowe na obrazie wejsciowym. Jest to jakby nakladanie na obraz
**     wyjsciowy znakow obecnych w obrazie wejsciowym. Obraz wyjsciowy
**     NIE JEST ZEROWANY. Piksle obrazu wyjsciowego, odpowiadajace nezerowym
**     pikslom obrazu wejsciowego, otrzymuja jasnosc z obrazu wejsciowego.
**     Pozostale piksle obrazu wyjsciowego nie zmieniaja jasnosci.
**       Obydwa obrazy sa obrazami SZARYMI (GREpic).
**       Funkcja moze byc uzyta zarowno do zmiany rozdzielczosci i proporcji
**     obrazu (powiekszanie i pomniejszanie, w skalach roznych dla wierszy
**     i kolumn) jak i dla przesuniec (obecna wersja umozliwia przesuniecia
**     o skonczona liczbe piksli).
**       W procesie skalowania wykorzystuje sie wylacznie operacje staloprze-
**     cinkowe. Dane staloprzecinkowe przygotowywane sa w tablicach Rshr, Cshr
**     przed skalowaniem, w operacjach zmiennoprzecinkowych, ktorych liczba
**     zalezy liniowo od wymiarow okien.
**       Liczby rzeczywiste (ulamki) symuluje sie liczbami calkowitymi,
**     powstalymi przez bitowe przesuniecie w lewo (shift left) liczb
**     ulamkowych. Przesuniecie o 8 bitow zapewnia dokladnosc 1/256, czyli
**     lepsza niz 0.5%. Przed skalowaniem powrotnym (>>8) do wynikow dodaje
**     sie reprezentacje 0.5, czyli 256/2 = 0x7F (bit 01111111), co prowadzi
**     do prawidlowego zaokraglania do calosci. Uzywane jest rowniez podwojne
**     skalowanie powrotne - po dwoch mnozeniach: dodaje sie wtedy (256^2)/2 =
**     0x7FFF (bit 01111111 11111111), i przesuwa >>16.
**
**     Funkcja korzysta z pamieci dynamicznej o wielkosci
**     ((XRout-MRout+1) + (XCout-MCout+1)) * sizeof(Tshr), gdzie
**     Tpixshr jest typem elementu tablicy, w ktorej zapisywane sa udzialy
**     liniowe (SHaRe) piksli okna wejsciowego w pikslu okna wyjsciowego.
**     sizeof(Tshr) = 6 bajtow.
**
**       Jesli ktores okno jest puste, lub okno ktoregos obrazu nie miesci
**     sie w nim, zwracany jest kod bledu.
**       Alokacja obrazu wyjsciowego nalezy do funkcji wywolujacej!
**
**     WARTOSCI ZWRACANE:
**     OK, BAD_DATA, NO_MEMORY.
**
**     FUNKCJE ZEWNETRZNE:
**     FindShares(),Find1Share(); (picscal.h, libpic.a)
**       Wypelnia tablice udzialow Ishr[] dla wspolrzednej I.
*/
/* definicje pomocnicze dla ResizePic() */
typedef struct {
  int           Iend,Ista;
  unsigned char shrend,shrsta;
} Tshr;
/* przeliczanie double z przedzialu [0..1] na byte */
#define d2b(x)    ( (byte)qround((x) * 256.0) )
void  FindShares(Tshr Ishr[],int MIinp,int XIinp,int MIout,int XIout,int mod);
unsigned long int Find1Share(int MIinp,int XIinp,int MIout,int XIout);


/*  Typy i makra do staloprzecinkowego SKALOWANIA WSPOLRZEDNYCH
    z zakresu 'inp' do zakresu 'out'.

  Na poczatku zostanie omowione proste skalowanie liniowe z zakresu do
zakresu: zakres = RaNGe (RNG).
  Skalowanie spelnia warunki (I to wspolrzedna - R lub C):
1. MIinp przechodzi w MIout, a dla MIinp-1 jest Iout < MIout.
2. XIinp przechodzi w XIout, a dla XIinp+1 jest Iout > XIout.
3. Nie jest zapewnione, ze np. XIinp+1 przechodzi w XIout+1 (patrz przyklad).
A zatem cechy topologiczne (rozlacznosci, dopelniania, ...) zakresow wejsciowych
przenosza sie na zakresy wyjsciowe.

  Skalowanie wykonuje sie tak, jak to zapisano w formie makra:
#define rngsca3(I,fac,add,shf)  (int)( ((I)*(fac) + (add)) >> (shf) )
gdzie skalujemy wartosc I, a czynnik fac, skladnik add i przesuniecie shf
sa parametrami skalowania, reprezentujacymi skalowanie liniowe. Parametry
sa calkowite - typu 'Tauxsca' - a zastosowanie przesuniecia sprawia, ze
mniej znaczace cyfry parametrow 'fac' i 'add' reprezentuja czesci ulamkowe.
Dzieki temu unika sie obliczen zmiennoprzecinkowych.
  Znajdowanie tych parametrow realizuje funkcja:
int PrepRngSca3(int Minp,int Xinp,int Mout,int Xout,
                Tauxsca *Pfac,Tauxsca *Padd,int *Pshf);
Funkcja ta znajduje najwiekszy 'shf', na jaki pozwalaja zakresy 'inp' i
'out', dla zapewnienia maksymalnej dokladnosci, oraz odpowiednie 'fac'
i 'add'.
  Mozna zapamietywac te trzy parametry skalowania jednego zakresu w drugi
w strukturze typu 'Trngscainf' i poslugiwac sie funkcja PrepRngSca() i
makrem skalujacym rngsca(), co jest prostsze, i zalecane.

  Powyzszy opis wystarcza, o ile szerokosc zakresu 'inp' jest wieksza,
niz zakresu 'out', gdyz wtedy kilka piksli przechodzi w jeden. Jesli zas
jest przeciwnie, to jeden piksel przechodzi w kilka piksli. Dlatego
przygotowano rowniez funkcje obliczajace PRZEDZIAL indeksow - od Minimum (M)
do maXimum (X) - zakresu 'out', w jaki przechodzi jeden piksel zakresu 'inp',
oraz wartosc sreDnia (D) z M i X.
  Nazwy funkcji utworzono od slowa "wspolrzedne" - CooRDinates (CRD).
  Sa to: funkcja znajdowania parametrow PrepCrdSca() oraz makra skalowania:
Mcrdsca(), Xcrdsca() i Dcrdsca(), odpowiednio dla Minimow, maXimow i sreDniej
(oczywiscie, jesli zakres 'inp' jest wiekszy niz 'out', to te funkcje daja wyniki
identyczne, jak funkcje PrepRngSca() i rngsca() ). Parametry skalowania
zapamietuje sie w strukturze typu 'Tcrdscainf'.

  Dla uproszczenia operacji na dwoch wspolrzednych, czyli na oknach
(WINdows (WIN)) przygotowano funkcje PrepWinSca(), makra MRsca(), XRsca(),
DRsca(), MCsca(), XCsca(), DCsca() do skalowania odpowiednio rzedow (Rows)
i kolumn (Columns), oraz typ struktury do zapamietywania parametrow
'Twinscainf'.

  PRZYKLAD WYNIKOW OBLICZEN 1          PRZYKLAD WYNIKOW OBLICZEN 2
Przeliczamy zakres [1..8] do [1..3]    Przeliczamy zakres [1..3] do [1..8]
        Mcrd rng Xcrd                          Mcrd rng Xcrd
8: 1 ->   1   1   1   :3               3: 1 ->   1   2   2   :8**)
   2 ->   1   1   1                       2 ->   4   4*) 5
   3 ->   1   1   1                       3 ->   7   7   8
   4 ->   2   2   2
   5 ->   2   2   2               *) Rownie dobrze mogloby byc 5.
   6 ->   3   3   3              **) Piksel 3 z przedzialu [1..8] przynalezy
   7 ->   3   3   3                  czesciowo do piksli 1 i 2 z przedzialu
   8 ->   3   3   3                  [1..3]; podobnie piksel 6 do piksli 2 i 3.


  PRZYKLAD PROGRAMU 1
  /# Chcemy przeliczac jasnosci 'brh' z zakresu 0..255 do 1..14 #/
  long int fac,add;
  int      shf;
  unsigned char brh1,brh2;

  if (PrepRngSca3(0,255,1,14,&fac,&add,&shf) != OK)
    exit(-1); /# ktorys z bledow #/

  /# przeliczenie #/
  brh2 = rngsca3(brh1,fac,add,shf);

  PRZYKLAD PROGRAMU 2
  /# Chcemy przeliczac wspolrzedne z okna win1 do win2 #/
  Twin win1,win2;
  Twinscainf sca12w; /# zmienne pomocnicze - informacje o skalowaniu #/
  Tcrdscainf sca12R;
  int MR2,XR2;
  /# tu zapiszemy ramke WOKOL pewnego okna w win1, przeliczonego do win2 #/
  Twin fra;

  if (PrepCrdSca(win1.MR,win1.XR,win2.MR,win2.XR,&sca12R) != OK)
    exit(-1); /# ktorys z bledow #/
  if (PrepWinSca(&win1,&win2,&sca12w) != OK)
    exit(-2);

  /#
  zwykle przeliczenia:
  "gorny" i "dolny" wiersz piksla okna 2, odpowiadajacego pikslowi 7 okna 1
  #/
  MR2 = Mcrdsca(7,sca12R);
  XR2 = Xcrdsca(7,sca12R);

  /# ramka WOKOL okna {6,3,15,17} w win1, przeliczona do win2 #/
  fra.MR = MRsca( 6,sca12w) - 1;
  fra.MC = MCsca( 3,sca12w) - 1;
  fra.XR = XRsca(15,sca12w) + 1;
  fra.XC = XCsca(17,sca12w) + 1;
*/

typedef signed long int Tauxsca;
#define Uauxsca (double)LONG_MAX
#define Wauxsca (double)LONG_MIN
typedef struct {
  Tauxsca fac,add;
  int     shf;
} Trngscainf;
typedef struct {
  Trngscainf M,X;
} Tcrdscainf;
typedef struct {
  Tcrdscainf x,y;
} Twinscainf;

/* Preparing the scaling: for 1 and 2 coordinates, and a window */
/* ATTENTION: link the MATHEMATICAL library libm.a to your program! */
int PrepRngSca3(int Minp,int Xinp,int Mout,int Xout,
  Tauxsca *Pfac,Tauxsca *Padd,int *Pshf);
int PrepRngSca(int Minp,int Xinp,int Mout,int Xout,
  Trngscainf *Pinf);
int PrepCrdSca(int Minp,int Xinp,int Mout,int Xout,
  Tcrdscainf *Pinf);
int PrepWinSca(const Twin *Pinpwin,const Twin *Poutwin,
  Twinscainf *Psca);
/*
WARTOSCI ZWRACANE: OK, BAD_DATA.
Funkcja PrepRngSca3() zraca dodatkowo COORINATE_DEGENERATE, jesli probujemy
przeskalowac zakres jednopikslowy w zakres wielopikslowy. Dla pozostalych
funkcji przypadek ten nie jest bledny.
*/

/* makro skalujace jedna wspolrzedna, z trzema parametrami i z jednym */
#define rngsca3(I,fac,add,shf)  (int)( ((I)*(fac) + (add)) >> (shf) )
#define rngsca(I,rnginf)  \
  (int)( ((I)*((rnginf).fac) + (rnginf).add) >> (rnginf).shf )

#define Mcrdsca(I,crdinf)  \
  (int)( ((I)*((crdinf).M.fac) + (crdinf).M.add) >> (crdinf).M.shf )
#define Xcrdsca(I,crdinf)  \
  (int)( ((I)*((crdinf).X.fac) + (crdinf).X.add) >> (crdinf).X.shf )
#define Dcrdsca(I,crdinf)  \
  ((Mcrdsca(I,crdinf) + Xcrdsca(I,crdinf)) >> 1)

#define Mxsca(I,wininf)  \
  (int)( ((I)*((wininf).x.M.fac) + (wininf).x.M.add) >> (wininf).x.M.shf )
#define Xxsca(I,wininf)  \
  (int)( ((I)*((wininf).x.X.fac) + (wininf).x.X.add) >> (wininf).x.X.shf )
#define Dxsca(I,wininf)  \
  ((Mxsca(I,wininf) + Xxsca(I,wininf)) >> 1)

#define Mysca(I,wininf)  \
  (int)( ((I)*((wininf).y.M.fac) + (wininf).y.M.add) >> (wininf).y.M.shf )
#define Xysca(I,wininf)  \
  (int)( ((I)*((wininf).y.X.fac) + (wininf).y.X.add) >> (wininf).y.X.shf )
#define Dysca(I,wininf)  \
  ((Mysca(I,wininf) + Xysca(I,wininf)) >> 1)

#endif /* !defined(PICSCAL_INCLUDED) */
