
/*
** WRITING/READING A PICTURE TO/FROM A FILE in TIFF v. 5.00 format
**
** Library name:                 picio
** This header file name:        pic_io.h
** Other necessary libraries:    pic tiff
** Other necessary header files: pic.h
*/

#if !defined(PIC_IO_INCLUDED)
  #define PIC_IO_INCLUDED

#ifdef __cplusplus
extern "C" {
#endif

#define TIFF_NOCOMPRESS                    0
#define TIFF_LZWCOMPRESS                   1
#define TIFF_MINISBLACK                    2
#define TIFF_MINISWHITE                    3

/* error codes returned by functions */
#define TIFF_FILE_NOT_FOUND              -11
#define TIFF_BAD_FILE_FORMAT             -12
/* all other error codes are inherited from pic.h */

/* function returning pointers to strings with error descriptions */
char * pic_ioErrStr(int err);


int ReadTiff(Tpic *Ppic, const char *Fname, char **description);
/* version with shorter arguments list */
int ReadTiffShort(Tpic *Ppic, const char *Fname);
/* Reads from file named Fname a picture in TIFF v. 5.00 to a structure
*Ppic. *Ppic does not need to be empty; it is freed before allocating
according to the contents of the TIFF file.
Description (if any) is returned in *description, which must be NULL on
entry to the function. Memory for this text is allocated by malloc() within
ReadTiff(). User is responsible for freeing this memory after use.
  Only 1,2,4 or 8 bits/pixel grey images and 8 bits/pixel RGB images are read.
  Returned error codes:
TIFF_FILE_NOT_FOUND  - Fname can not be opened;
TIFF_BAD_FILE_FORMAT - not a TIFF v. 5.00 file; bad bits/pix number.
BAD_DATA             - *description not NULL.
NO_MEMORY            - no memory for picture line buffer.
READ_ERROR           - error while reading pixels.
  In case of error *Ppic may contain some or all bad data. It is safe to
do FreePic(Ppic) then.
*/


int WriteTiff(const char *Fname, Tpic *Ppic, Twin *Pwin,
  int compression, int whatismin, char *description);
/* version with shorter arguments list */
#define WriteTiffShort(Fname,Ppic) \
  WriteTiff(Fname,Ppic,NULL,TIFF_LZWCOMPRESS,TIFF_MINISBLACK,NULL)

/* Writes window *Pwin of image *Ppic to file named *Fname.
compression: TIFF_NOCOMPRESS, TIFF_LZWCOMPRESS.
whatismin:   TIFF_MINISWHITE, TIFF_MINISBLACK (not meaningful for colour images).
  Returned error codes:
TIFF_FILE_NOT_FOUND  - Fname can not be opened;
BAD_DATA             - *Ppic mode unknown; *Pwin degenerated or out of *Ppic;
                       bad value of compression or whatismin (if not
                       a colour picture).
NO_MEMORY            - no memory for picture line buffer.

  Line buffer size is set so that it takes not more than 8KB memory.
  Field RESOLUTIONUNIT is set to NONE.
*/

#ifdef __cplusplus
}
#endif

#endif /* !defined(PIC_IO_INCLUDED) */
