
#include "../tiff/tiffio.h"
#include "../tiff/tiff.h"
#include "pic.h"
#include "pic_io.h"



int ReadTiffShort(Tpic *Ppic, const char *Fname) {
  char *txt = NULL;
  int err;

  err = ReadTiff(Ppic,Fname,&txt);
  Delete(txt);

  return err;
} /* ReadTiffShort */



int ReadTiff(Tpic *Ppic, const char *Fname, char **description)
{
            TIFF*   tif;
   TIFFDirectory*   td;
          u_short   bits,config,width,height,photom,samples;
             long   header;
	      int   row,col,pom;
    unsigned char   *buf,*bufR,*bufG,*bufB, *msk=NULL, dana;
	     char   pom_c;
	    FILE*   fl;

    config=PLANARCONFIG_CONTIG;
    fl=fopen(Fname,"rb");
    if (fl==NULL)
      return(TIFF_FILE_NOT_FOUND);
    header=getw(fl);
    fclose(fl);
    if (header!=0x002A4949 && header!=0x2A004D4D && header !=0x49492a00)
       return(TIFF_BAD_FILE_FORMAT);
    if (*description!=NULL)
       return(BAD_DATA);

    if ( (tif = TIFFOpen(Fname,"rb")) == NULL) return(TIFF_FILE_NOT_FOUND);
    td=&(tif->tif_dir);
    width=td->td_imagewidth;
    if (width<1) {
      TIFFClose(tif);
      /*printf("\n 2");*/
      return(TIFF_BAD_FILE_FORMAT);
    }
    height=td->td_imagelength;
    if (height<1 ) {
      TIFFClose(tif);
      /*printf("\n 3");*/
      return(TIFF_BAD_FILE_FORMAT);
    }
    if (td->td_imagedescription != NULL)
      pom=strlen(td->td_imagedescription)+1;
    else
      pom=0;
    if (pom>1)
    {  *description=(char*)malloc((unsigned long)pom);
       if (*description==NULL) {  TIFFClose(tif); return(NO_MEMORY);}
       memcpy(*description,td->td_imagedescription,(unsigned)pom);
    }
    samples=td->td_samplesperpixel;
    if (samples!=1 && samples!=3) {
      TIFFClose(tif);
      /*printf("\n 4");*/
      return(TIFF_BAD_FILE_FORMAT);
    }
    bits=td->td_bitspersample;
    if (samples==1 && !(bits==1 || bits==2 || bits==4 || bits==8)) {
      TIFFClose(tif);
      /*printf("\n 5");*/
      return(TIFF_BAD_FILE_FORMAT);
    }
    if (samples==3 && !(bits==4 || bits==8)) {
      TIFFClose(tif);
      /*printf("\n 6");*/
      return(TIFF_BAD_FILE_FORMAT);
    }
    switch (samples)
    {  case 1:  /* obraz monochromatyczny */
		switch (bits) {
		  case 1: /* obraz binarny */
		    if ( (msk=(unsigned char*)malloc(sizeof(unsigned char)*8))==NULL) {
		      TIFFClose(tif);
                      return(NO_MEMORY);
		    }
                    if (AllocPic(Ppic,BINpic,width,height) != OK) {
		      TIFFClose(tif);
		      free(msk);
                      return(NO_MEMORY);
		    }
		    msk[0]=(unsigned char)128;
		    msk[1]=(unsigned char)64;
		    msk[2]=(unsigned char)32;
		    msk[3]=(unsigned char)16;
		    msk[4]=(unsigned char)8;
		    msk[5]=(unsigned char)4;
		    msk[6]=(unsigned char)2;
		    msk[7]=(unsigned char)1;
		    break;
		  case 2:  /* obraz dwubitowy */
		    if ( (msk=(unsigned char*)malloc(sizeof(unsigned char)*4))==NULL) {
		      TIFFClose(tif);
                      return(NO_MEMORY);
		    }
                    if (AllocPic(Ppic,BIT2pic,width,height) != OK) {
		      TIFFClose(tif);
		      free(msk);
                      return(NO_MEMORY);
		    }
		    msk[0]=(unsigned char)192;
		    msk[1]=(unsigned char)48;
		    msk[2]=(unsigned char)12;
		    msk[3]=(unsigned char)3;
		    break;
		  case 4:  /* obraz czterobitowy */
		    if ( (msk=(unsigned char*)malloc(sizeof(unsigned char)*2))==NULL) {
		      TIFFClose(tif);
                      return(NO_MEMORY);
		    }
                    if (AllocPic(Ppic,BIT4pic,width,height) != OK) {
		      TIFFClose(tif);
		      free(msk);
                      return(NO_MEMORY);
		    }
		    msk[0]=(unsigned char)240;
		    msk[1]=(unsigned char)15;
		    break;
		  case 8:  /* obraz osmiobitowy */
                    if (AllocPic(Ppic,GREpic,width,height) != OK) {
		      TIFFClose(tif);
                      return(NO_MEMORY);
		    }
		    break;
		}
		photom=td->td_photometric;
		photom=(photom==PHOTOMETRIC_MINISBLACK)?1:0;
                config=PLANARCONFIG_CONTIG;
		break;
       case 3:  /* obraz kolorowy */
                if (AllocPic(Ppic,CLRpic,width,height) != OK)
                {  TIFFClose(tif);  return(NO_MEMORY);}
                config=td->td_planarconfig;
                if (config!=PLANARCONFIG_CONTIG &&
                                         config!=PLANARCONFIG_SEPARATE) {
                  TIFFClose(tif);
                  /*printf("\n 7");*/
                  return(TIFF_BAD_FILE_FORMAT);
                }
		break;
       default: TIFFClose(tif);
                /*printf("\n 8");*/
                return(TIFF_BAD_FILE_FORMAT);
    }
    if (td->td_stripsperimage==1L &&
	  td->td_rowsperstrip!=(u_long)(td->td_imagelength) )
	td->td_rowsperstrip=(u_long)(td->td_imagelength);
    if ( config == PLANARCONFIG_CONTIG )
    {  buf=(unsigned char*)malloc((unsigned long)TIFFScanlineSize(tif));
       if (buf==NULL) {
	 TIFFClose(tif);
	 if (msk!=NULL) free(msk);
         return(NO_MEMORY);
       }
       for (row=0;row<=height-1;row++) {
           if (TIFFReadScanline(tif,buf,(unsigned)row,0)<0) {
	     TIFFClose(tif);
	     if (msk!=NULL) free(msk);
             return(READ_ERROR);
	   }
	   switch (samples)
	   {   case 1: /* obraz monochromatyczny */
		 switch (bits) {
		   case 1: /* obrazy binarne */
                     for (col=0;col<width;col++) {
		       dana= ( buf[col/8] & msk[col%8] ) >> ( 7 - (col%8) );
                       SetBIN(Ppic,col,row,(Tbrh)dana);
		     }
		     break;
		   case 2: /* obrazy dwubitowe */
                     for (col=0;col<width;col++) {
		       dana= ( buf[col/4] & msk[col%4] ) >> ( 6 - (col%4)*2 );
                       SetBIT2(Ppic,col,row,(Tbrh)dana);
		     }
		     break;
		   case 4: /* obrazy czterobitowe */
                     for (col=0;col<width;col++) {
		       dana= ( buf[col/2] & msk[col%2] ) >> ( 4 - (col%2)*4 );
                       SetBIT4(Ppic,col,row,(Tbrh)dana);
		     }
		     break;
		   case 8: /* obrazy osmiobitowe */
                     for (col=0;col<width;col++)
                       SetGRE(Ppic,col,row,(Tbrh)buf[col]);
		     break;
		 }
		 break;
               case 3:
                if (bits==8)
                  for (col=0;col<width;col++)
                  {  SetCLR1(Ppic,col,row,Rclr,buf[3*col]);
                     SetCLR1(Ppic,col,row,Gclr,buf[3*col+1]);
                     SetCLR1(Ppic,col,row,Bclr,buf[3*col+2]);
                  }
                else
                  for (col=0;col<TIFFScanlineSize(tif);col=col+3)
                  {   pom_c=buf[col];
                      SetCLR1(Ppic,(2*col)/3,row,Rclr,pom_c>>4);
                      SetCLR1(Ppic,(2*col)/3,row,Gclr,buf[col] & 0x0F);
                      pom_c=buf[col+1];
                      SetCLR1(Ppic,(2*col)/3,row,Bclr,pom_c>>4);
                      if ( ((2*col)/3+1)<width )
                      {  SetCLR1(Ppic,(2*col)/3+1,row,Rclr,buf[col+1] & 0x0F);
                         pom_c=buf[col+2];
                         SetCLR1(Ppic,(2*col)/3+1,row,Gclr,pom_c>>4);
                         SetCLR1(Ppic,(2*col)/3+1,row,Bclr,buf[col+2] & 0x0F);
                      }
                  }
	   }
       }
       free(buf);
       if (msk==NULL) free(msk);
    }
    else
    {  bufR=(unsigned char*)malloc((unsigned long)TIFFScanlineSize(tif));
       if (bufR==NULL)  { TIFFClose(tif); return(NO_MEMORY);}
       bufG=(unsigned char*)malloc((unsigned long)TIFFScanlineSize(tif));
       if (bufG==NULL)  { TIFFClose(tif); return(NO_MEMORY);}
       bufB=(unsigned char*)malloc((unsigned long)TIFFScanlineSize(tif));
       if (bufB==NULL)  { TIFFClose(tif); return(NO_MEMORY);}
       for (row=0;row<=height-1;row++)
       {   if (TIFFReadScanline(tif,bufR,(unsigned)row,0)<0)
           {  TIFFClose(tif); return(READ_ERROR);}
           if (TIFFReadScanline(tif,bufG,(unsigned)row,1)<0)
           {  TIFFClose(tif); return(READ_ERROR);}
           if (TIFFReadScanline(tif,bufB,(unsigned)row,2)<0)
           {  TIFFClose(tif); return(READ_ERROR);}
	   if (bits==8)
              for (col=0;col<width;col++)
              {  SetCLR1(Ppic,col,row,Rclr,bufR[col]);
                 SetCLR1(Ppic,col,row,Gclr,bufG[col]);
                 SetCLR1(Ppic,col,row,Bclr,bufB[col]);
	      }
	   else
	      for (col=0;col<TIFFScanlineSize(tif);col++)
	      {  pom_c=bufR[col];
                 SetCLR1(Ppic,2*col,row,Rclr,pom_c>>4);
                 if ( (2*col+1)<width )
                    SetCLR1(Ppic,2*col+1,row,Rclr,bufR[col] & 0x0F);
		 pom_c=bufG[col];
                 SetCLR1(Ppic,2*col,row,Gclr,pom_c>>4);
                 if ( (2*col+1)<width )
                    SetCLR1(Ppic,2*col+1,row,Gclr,bufG[col] & 0x0F);
		 pom_c=bufB[col];
                 SetCLR1(Ppic,2*col,row,Bclr,pom_c>>4);
                 if ( (2*col+1)<width )
                    SetCLR1(Ppic,2*col+1,row,Bclr,bufR[col] & 0x0F);
	      }
       }
       free(bufR);
       free(bufG);
       free(bufB);
    }
    TIFFClose(tif);
    return(OK);
} /* ReadTiff */



int WriteTiff(const char *Fname, Tpic *Ppic, Twin *Pwin,
  int compression, int whatismin, char *description)
{
         TIFF*   tif;
       u_short   bits,height,width,samples,photom=whatismin;
           int   row,col,pom,Mx,My;
 unsigned long   lpom;
          long   llpom = 0L;
 unsigned char   *line = NULL, dana, *msk=NULL;

  if (
    _mod(Ppic) != BINpic &&
    _mod(Ppic) != BIT2pic &&
    _mod(Ppic) != BIT4pic &&
    _mod(Ppic) != GREpic &&
    _mod(Ppic) != CLRpic
  ) return(BAD_DATA);
  switch (compression) {
  case TIFF_NOCOMPRESS:
    compression = COMPRESSION_NONE;
    break;
  case TIFF_LZWCOMPRESS:
    compression = COMPRESSION_LZW;
    break;
  default:
    return(BAD_DATA);
  }
  if (_mod(Ppic) != CLRpic) switch (whatismin) {
  case TIFF_MINISBLACK:
    whatismin = PHOTOMETRIC_MINISBLACK;
    break;
  case TIFF_MINISWHITE:
    whatismin = PHOTOMETRIC_MINISWHITE;
    break;
  default:
    return(BAD_DATA);
  }
  if (Pwin == NULL) {
    width=Ppic->Zx; height=Ppic->Zy;
    Mx = 0; My = 0;
  } else {
    if (!IsWinOK(Pwin) || !IsWinInPic(Pwin,Ppic))
      return(BAD_DATA);
    width=Pwin->Zx; height=Pwin->Zy;
    Mx = Pwin->Mx; My = Pwin->My;
  }
  pom=height;
  if ((tif = TIFFOpen(Fname,"wb"))==NULL) return(TIFF_FILE_NOT_FOUND);
  TIFFSetField(tif, TIFFTAG_IMAGEWIDTH, width);
  TIFFSetField(tif, TIFFTAG_IMAGELENGTH, height);
  bits=8;
  switch ( _mod(Ppic) ) {
    case BINpic  : bits=1; break;
    case BIT2pic : bits=2; break;
    case BIT4pic : bits=4; break;
    case GREpic  : bits=8; break;
    case CLRpic  : bits=8; break;
  }
  TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, bits);
  TIFFSetField(tif, TIFFTAG_COMPRESSION, compression);
  if ( _mod(Ppic) != CLRpic )
  {  samples=1;
     photom=whatismin;
     switch (bits) {
       case 1: /* obraz binarny */
         line=(unsigned char*)malloc((unsigned long)(width/8 + ( (width%8)?1:0 ) ));
	 break;
       case 2: /* obraz dwubitowy */
         line=(unsigned char*)malloc((unsigned long)(width/4 + ( (width%4)?1:0 ) ));
	 break;
       case 4: /* obraz czterobitowy */
         line=(unsigned char*)malloc((unsigned long)(width/2 + ( (width%2)?1:0 ) ));
	 break;
       case 8: /* obraz osmiobitowy */
         line=(unsigned char*)malloc((unsigned long)width);
	 break;
     }
     if (line==NULL)
     {  TIFFClose(tif);  return(NO_MEMORY); }
     switch (bits) {
       case 1: llpom= (long)width/8L + ( ((long)width%8L)?1L:0L ); break;
       case 2: llpom= (long)width/4L + ( ((long)width%4L)?1L:0L ); break;
       case 4: llpom= (long)width/2L + ( ((long)width%2L)?1L:0L ); break;
       case 8: llpom= (long)width; break;
     }
     pom=height;
     while ( (long)pom*llpom > 8192L )
	   pom>>=1;
  }
  else
  {  samples=3;
     photom=PHOTOMETRIC_RGB;
     line=(unsigned char*)malloc((unsigned long)(width)*3L);
     if (line==NULL)
     {  TIFFClose(tif);  return(NO_MEMORY); }
     while ( (long)pom*(long)(width)*3L > 8192L )
       pom>>=1;
  }
  if (pom<1)
    pom=1;
  lpom=(unsigned long)pom;
  TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, photom);
  TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, samples);
  TIFFSetField(tif, TIFFTAG_ROWSPERSTRIP, lpom);
  TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
  TIFFSetField(tif, TIFFTAG_RESOLUTIONUNIT, RESUNIT_NONE);
  if (description!= NULL && strlen(description))
     TIFFSetField(tif, TIFFTAG_IMAGEDESCRIPTION, description);
  TIFFSetField(tif, TIFFTAG_SOFTWARE, "AIP Picture IO Lib");

  switch (bits) {
    case 1: if ( (msk=(unsigned char*)malloc(sizeof(unsigned char)*8))==NULL ) {
              free(line);
              TIFFClose(tif);
              return(NO_MEMORY);
            }
            msk[0]=127;
            msk[1]=191;
            msk[2]=223;
            msk[3]=239;
            msk[4]=247;
            msk[5]=251;
            msk[6]=253;
            msk[7]=254;
            break;
    case 2: if ( (msk=(unsigned char*)malloc(sizeof(unsigned char)*4))==NULL ) {
              free(line);
              TIFFClose(tif);
              return(NO_MEMORY);
            }
            msk[0]=63;
            msk[1]=207;
            msk[2]=243;
            msk[3]=252;
            break;
    case 4: if ( (msk=(unsigned char*)malloc(sizeof(unsigned char)*2))==NULL ) {
              free(line);
              TIFFClose(tif);
              return(NO_MEMORY);
            }
            msk[0]=15;
            msk[1]=240;
            break;
  }
  for (row=0; row<height; row++)
  {   if ( _mod(Ppic) != CLRpic ) {
        switch (bits) {
          case 1: /* obraz monochromatyczny */
            for (col=0;col<width;col++) {
              dana = (unsigned char)BIN(Ppic,col+Mx,row+My);
              dana = dana << ( 7 - (col)%8 ) ;
              line[(col)/8]&=msk[(col)%8];
              line[(col)/8]|=dana;
            }
            break;
          case 2: /* obraz dwubitowy */
            for (col=0;col<width;col++) {
              dana = (unsigned char)BIT2(Ppic,col+Mx,row+My);
              dana = dana << ( (3-(col)%4) << 1 ) ;
              line[(col)/4]&=msk[(col)%4];
              line[(col)/4]|=dana;
            }
            break;
          case 4: /* obraz czterobitowy */
            for (col=0;col<width;col++) {
              dana = (unsigned char)BIT4(Ppic,col+Mx,row+My);
              dana = dana << ( (1- (col)%2) << 2 ) ;
              line[(col)/2]&=msk[(col)%2];
              line[(col)/2]|=dana;
            }
            break;
          case 8: /* obraz osmiobitowy */
            for (col=0;col<width;col++) {
              dana = (unsigned char)GRE(Ppic,col+Mx,row+My);
              line[col]=dana;
            }
            break;
        }
      }
      else
        for (col=0;col<width;col++)
        {  *(line+3*(col)  ) = CLR1(Ppic,col+Mx,row+My,Rclr);
           *(line+3*(col)+1) = CLR1(Ppic,col+Mx,row+My,Gclr);
           *(line+3*(col)+2) = CLR1(Ppic,col+Mx,row+My,Bclr);
        }
      TIFFWriteScanline(tif,line,(unsigned)row,0);
  }

  free(line);
  TIFFClose(tif);
  if (msk!=NULL) free(msk);
  return(OK);
} /* WriteTiff */
