/* * * * * * * * * * * * * * * * * * * * * * * * * * * *
*  Module morfop.c                                     *
*  Copyright (C) 1988 by Mariusz Nieniewski            *
*                                                      *
*  Permission is granted to use and copy this package, *
*  but not to sell any part of if without the author's *
*  written permission. This notice must remain in  any *
*  copy or derivative.                                 *
*                                 mnieniew@ippt.gov.pl *
*                                   nieniew&iel.waw.pl *
* * * * * * * * * * * * * * * * * * * * * * * * * * * */ 

/* Module morfop.c  contains the functions carrying out the
   following transformations:
   dilation, erosion, opening, closing,
   subtraction of images,
   minimum of two images, maximum of two images, thresholding,
   sampling, reconstruction, blocking.
   Created on Mar 27, 1997 */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <malloc.h>
#include "pic.h"
#include "pic_io.h"
#include "morf.h"

extern int UR;
extern int UC;
extern int UR1;
extern int UC1;
extern int UR_reduced;
extern int UC_reduced;
extern int UR1_reduced;
extern int UC1_reduced;
extern Tpic ypic;
extern Tpic * Pypic;
extern Tpic ipic[3],opic[3],wpic[3];

char txtbuf[30];
int tmpfnstr[MAXSIZEFNSTR];

unsigned char one_pixel_dil(int pic_col, int pic_row,
      Tpic * picture,
      TPfnstr funstr)
{
  int sum_row, sum_col, sum_val;
  int no_point_funstr = 0;
  int local_value, local_maximum;
  unsigned char out_val;

  local_maximum = 0;
  while ((*funstr)[no_point_funstr] != FileEmptyPoint) {
    sum_col = pic_col - (*funstr)[no_point_funstr];
    sum_row = pic_row + (*funstr)[no_point_funstr+1];
    sum_val = (*funstr)[no_point_funstr+2];

    if (sum_row >= 0 && sum_row <= UR1 &&
        sum_col >= 0 && sum_col <= UC1 )
      local_value = GRE(picture,sum_col,sum_row) + sum_val;
    else
      local_value=-1;
    if (local_value > local_maximum) {
      local_maximum = local_value;
      if (local_maximum > 255) 
        local_maximum = 255;
    }
    no_point_funstr += 3;
  } /* end of while */

  out_val = (unsigned char)local_maximum;
  return(out_val) ;
} /* END of one_pixel_dil */

int f_one_pixel_dil(int pic_col, int pic_row,
      Tpic * picture,
      TPfnstr funstr)
{
  int sum_row, sum_col, sum_val;
  int no_point_funstr = 0;
  int val,local_value, local_maximum;
  int out_val;

  local_maximum = -255;
  while ((*funstr)[no_point_funstr] != FileEmptyPoint) {
    sum_col = pic_col - (*funstr)[no_point_funstr];
    sum_row = pic_row + (*funstr)[no_point_funstr+1];
    sum_val = (*funstr)[no_point_funstr+2];

    if (sum_row >= 0 && sum_row <= UR1 &&
        sum_col >= 0 && sum_col <= UC1 )
      {
       if((val=GRE(picture+1,sum_col,sum_row))==0)
         local_value=-GRE(picture,sum_col,sum_row);
       else if(val==255)
         local_value=GRE(picture+2,sum_col,sum_row)+255;
       else local_value=val;
       local_value+=sum_val;
      }
    else
      local_value=-255;
    if (local_value > local_maximum) local_maximum = local_value;
    no_point_funstr += 3;
  } /* end of while */

  out_val = local_maximum;
  return(out_val) ;
} /* END of f_one_pixel_dil */


unsigned char one_pixel_ero(int pic_col, int pic_row,
      Tpic * picture,
      TPfnstr funstr)
{
  int sum_row, sum_col, sum_val;
  int no_point_funstr = 0;
  int local_value, local_minimum;
  /*int tmp_cntr = 0;*/
  unsigned char out_val;

  local_minimum = 255;
  while ((*funstr)[no_point_funstr] != FileEmptyPoint) {
    sum_col = pic_col + (*funstr)[no_point_funstr];
    sum_row = pic_row - (*funstr)[no_point_funstr+1];
    sum_val = (*funstr)[no_point_funstr+2];

    if (sum_row >= 0 && sum_row <= UR1 &&
        sum_col >= 0 && sum_col <= UC1)
      local_value = GRE(picture,sum_col,sum_row) - sum_val;
    else
      local_value = 255;
    if (local_value < local_minimum) {
      local_minimum = local_value;
      if (local_minimum < 0) 
        local_minimum = 0;
    }
    no_point_funstr += 3;
  } /* end of while */

  out_val = (unsigned char)local_minimum;
  return(out_val);
} /* END of one_pixel_ero */

int f_one_pixel_ero(int pic_col, int pic_row,
      Tpic * picture,
      TPfnstr funstr)
{
  int sum_row, sum_col, sum_val;
  int no_point_funstr = 0;
  int val,local_value, local_minimum;
  int out_val;

  local_minimum = 511;
  while ((*funstr)[no_point_funstr] != FileEmptyPoint) {
    sum_col = pic_col + (*funstr)[no_point_funstr];
    sum_row = pic_row - (*funstr)[no_point_funstr+1];
    sum_val = (*funstr)[no_point_funstr+2];

    if (sum_row >= 0 && sum_row <= UR1 &&
        sum_col >= 0 && sum_col <= UC1)
      {
       if((val=GRE(picture+1,sum_col,sum_row))==0)
         local_value=-GRE(picture,sum_col,sum_row);
       else if(val==255)
         local_value=GRE(picture+2,sum_col,sum_row)+255;
       else local_value=val;
       local_value-=sum_val;
      }
    else
      local_value = 511;
    if (local_value < local_minimum)local_minimum = local_value;
    no_point_funstr += 3;
  } /* end of while */

  out_val = local_minimum;
  return(out_val);
} /* END of f_one_pixel_ero */


int dilation_c(Tpic * picture, Tpic * result, TPfnstr funstr)
{
  int pic_row, pic_col;
  for (pic_row = 0; pic_row <= UR1; ++pic_row) {
    for (pic_col = 0; pic_col <= UC1; ++pic_col)
      SetGRE(result,pic_col,pic_row,
      (Tbrh) one_pixel_dil(pic_col,pic_row,picture,(TPfnstr)funstr));
  }
  return OK;
} /*  END of dilation_c */

int f_dilation_c(Tpic * picture, Tpic * result, TPfnstr funstr)
{
  int pic_row, pic_col,val;
  unsigned char v;
  for (pic_row = 0; pic_row <= UR1; ++pic_row)
    for (pic_col = 0; pic_col <= UC1; ++pic_col)
    {
     val = f_one_pixel_dil(pic_col,pic_row,picture,(TPfnstr)funstr);
     if(val<=0){v=-val;SetGRE(result,pic_col,pic_row,(Tbrh)v);v=0;}
     else if(val>=255){v=val-255;SetGRE(result+2,pic_col,pic_row,(Tbrh)v);v=255;}
     else v=val;
     SetGRE(result+1,pic_col,pic_row,(Tbrh)v);
    }
  return OK;
} /*  END of f_dilation_c */

int erosion_c(Tpic * picture, Tpic * result, TPfnstr funstr)
{
  int pic_row, pic_col;

/*  printf("\n Erosion: row ");*/
  for (pic_row = 0; pic_row <= UR1; pic_row++) {
/*    printf("."); fflush(stdout); */
    /*    printf(" %3d", pic_row);*/
    for (pic_col = 0; pic_col <= UC1; pic_col++)
      SetGRE(result,pic_col,pic_row,
      (Tbrh) one_pixel_ero(pic_col,pic_row,picture,(TPfnstr)funstr));
  }
  return OK;
} /*  END of erosion_c */

int f_erosion_c(Tpic * picture, Tpic * result, TPfnstr funstr)
{
  int pic_row, pic_col,val;
  unsigned char v;
  for (pic_row = 0; pic_row <= UR1; pic_row++)
    for (pic_col = 0; pic_col <= UC1; pic_col++)
    {
     val = f_one_pixel_ero(pic_col,pic_row,picture,(TPfnstr)funstr);
     if(val<=0){v=-val;SetGRE(result,pic_col,pic_row,(Tbrh)v);v=0;}
     else if(val>=255){v=val-255;SetGRE(result+2,pic_col,pic_row,(Tbrh)v);v=255;}
     else v=val;
     SetGRE(result+1,pic_col,pic_row,(Tbrh)v);
    }
  return OK;
} /*  END of f_erosion_c */


int opening_c(Tpic * picture, Tpic * result, TPfnstr funstr)
{
  int ret_code = OK;
  
  Pypic=&ypic;
  AllocPic(Pypic,GREpic,UC,UR);
  ret_code = erosion_c(picture, Pypic, funstr);
  ret_code = dilation_c(Pypic, result, funstr);
  FreePic(Pypic);
  return ret_code;
} /*  END of opening_c */

int f_opening_c(Tpic * picture, Tpic * result, TPfnstr funstr)
{
  int ret_code = OK;

  AllocPic(&wpic[0],GREpic,UC,UR);
  AllocPic(&wpic[1],GREpic,UC,UR);
  AllocPic(&wpic[2],GREpic,UC,UR);
  ret_code = f_erosion_c(picture, &wpic[0], funstr);
  ret_code = f_dilation_c(&wpic[0], result, funstr);
  FreePic(&wpic[0]);
  FreePic(&wpic[1]);
  FreePic(&wpic[2]);
  return ret_code;
} /*  END of f_opening_c */

int closing_c(Tpic * picture, Tpic * result, TPfnstr funstr)
{
  int ret_code;

  Pypic=&ypic;
  AllocPic(Pypic,GREpic,UC,UR);
  ret_code = dilation_c(picture, Pypic, funstr);
  ret_code = erosion_c(Pypic, result, funstr);
  FreePic(Pypic);
  return ret_code;
} /*  END of closing_c */

int f_closing_c(Tpic * picture, Tpic * result, TPfnstr funstr)
{
  int ret_code = OK;
  AllocPic(&wpic[0],GREpic,UC,UR);
  AllocPic(&wpic[1],GREpic,UC,UR);
  AllocPic(&wpic[2],GREpic,UC,UR);
  ret_code = f_dilation_c(picture, &wpic[0], funstr);
  ret_code = f_erosion_c(&wpic[0], result, funstr);
  FreePic(&wpic[0]);
  FreePic(&wpic[1]);
  FreePic(&wpic[2]);
  return ret_code;
} /*  END of f_closing_c */


int maximum(Tpic * picture_1, Tpic * picture_2, Tpic * result)
{
  int pic_row, pic_col;

  for (pic_row = 0; pic_row <= UR1; pic_row++)
  for (pic_col = 0; pic_col <= UC1; pic_col++) {
    if (GRE(picture_1,pic_col,pic_row) < GRE(picture_2,pic_col,pic_row))
      SetGRE(result,pic_col,pic_row, GRE(picture_2,pic_col,pic_row));
    else 
      SetGRE(result,pic_col,pic_row, GRE(picture_1,pic_col,pic_row));
  }
  return OK;
} /* END of max */

int minimum(Tpic * picture_1, Tpic * picture_2, Tpic * result)
{
  int pic_row, pic_col;

  for (pic_row=0; pic_row <= UR1; pic_row++)
   for (pic_col=0; pic_col <= UC1; pic_col++) {
    if (GRE(picture_1,pic_col,pic_row) > GRE(picture_2,pic_col,pic_row))
      SetGRE(result,pic_col,pic_row, GRE(picture_2,pic_col,pic_row));
    else 
      SetGRE(result,pic_col,pic_row, GRE(picture_1,pic_col,pic_row));
  }
   
  return OK;
} /* END of minimum */

int mfsubtraction(Tpic * picture_1, Tpic * picture_2, Tpic * result, int mf)
{
  int pic_row, pic_col;
  unsigned int tmp;
  int BAD1, BAD2;
  
  BAD1=1;
  BAD2=2;
  for (pic_row = 0; pic_row <= UR1; pic_row++)
   for (pic_col = 0; pic_col <= UC1; pic_col++)
    if (GRE(picture_1,pic_col,pic_row) < GRE(picture_2,pic_col,pic_row))
      return BAD1;
    else {
      tmp= mf* ((unsigned int) (GRE(picture_1,pic_col,pic_row)
                            -GRE(picture_2,pic_col,pic_row)));
      if (tmp>255) 
        return BAD2;
      SetGRE(result,pic_col,pic_row, (Tbrh) tmp);
   }
   return OK;
} /* END of mfsubtraction */

int threshold(Tpic * picture, Tpic * result, int thr)
{
  int pic_row, pic_col;
  Tbrh inval, outval;

  for (pic_row = 0; pic_row <= UR1; pic_row++)
  for (pic_col = 0; pic_col <= UC1; pic_col++) {
    inval = GRE(picture,pic_col,pic_row);
    if (inval> thr) 
      outval=255; 
    else 
      outval=0;
    SetGRE(result,pic_col,pic_row, (Tbrh) outval);
  }
  return OK;
} /* end of threshold*/

int complement(Tpic * picture, Tpic * result)
{
  int pic_row, pic_col;
  int inval, outval;

  for (pic_row = 0; pic_row <= UR1; pic_row++)
  for (pic_col = 0; pic_col <= UC1; pic_col++) {
    inval = GRE(picture,pic_col,pic_row);
    outval = 255 - inval;
    SetGRE(result,pic_col,pic_row, (Tbrh) outval);
  }
  return OK;
}/* end of complement*/

int sample(Tpic * picture_1, Tpic * result)
{
  int row_out, col_out;
  int rr, rc;

  for (row_out = 0; row_out <= UR1_reduced; row_out++)
  for (col_out = 0; col_out <= UC1_reduced; col_out++) {
    rr=row_out;
    rc=col_out;
    SetGRE(result,rc,rr, GRE(picture_1,reduction*rc,reduction*rr));
  }
  return OK;
} /* END of sample */

int recons(Tpic * picture_1, Tpic * result)
{
  int ret_code = OK;
  int row_out, col_out;
  int rr, rc;

  ZeroPic(result);
  for (row_out = 0; row_out <= UR1_reduced; row_out++)
  for (col_out = 0; col_out <= UC1_reduced; col_out++) {
    rr=row_out*reduction;
    rc=col_out*reduction;
    SetGRE(result,rc,rr, (Tbrh) GRE(picture_1,col_out,row_out));
  }
  return(ret_code);
} /* END of recons */

int block(Tpic * picture_1, Tpic * result)
{
  int ret_code = OK;
  int row_out, col_out,i,j;

  ZeroPic(result);
  for (row_out = 0; row_out <= UR1_reduced; row_out++)
  for (col_out = 0; col_out <= UC1_reduced; col_out++)
   for (i=0; i<reduction; i++)
   for (j=0; j<reduction; j++)
    SetGRE(result,reduction*col_out+j,reduction*row_out+i, 
           (Tbrh) GRE(picture_1,col_out,row_out));


  return(ret_code);
} /* END of block */

