/* * * * * * * * * * * * * * * * * * * * * * * * * * * *
*  Module recgray.c                                    *
*  Copyright (C) 1988 by Mariusz Nieniewski            *
*                                                      *
*  Permission is granted to use and copy this package, *
*  but not to sell any part of if without the author's *
*  written permission. This notice must remain in  any *
*  copy or derivative.                                 *
*                                 mnieniew@ippt.gov.pl *
*                                   nieniew&iel.waw.pl *
* * * * * * * * * * * * * * * * * * * * * * * * * * * */ 

/* Module recgray.c - Reconstruction using queues of pixels
   Created by T. C. Nguyen on Apr 10, 1998 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "pic.h"
#include "pic_io.h"

extern int UR;
extern int UC;
extern int UR1;
extern int UC1;
extern int UR_reduced;
extern int UC_reduced;

typedef struct tagNode
{
 struct tagNode *next;
 int x,y;
} Node;
Node *start,*end,*tmp;
Node *Insert(int x,int y)
{
 Node *nd=(Node*)malloc(sizeof(Node));
 nd->x=x;nd->y=y;
 nd->next=NULL;
 if(end) end->next=nd;
 if(start==NULL)start=nd;//if start==NULL
 return nd;
}
Node *Remove(int *x,int *y)
{
 if(start)
  {
   Node *nd=start->next;
   if(nd==NULL)end=NULL;
   *x=start->x;*y=start->y;
   free(start);
   return nd;
  }
 else
  return NULL;
}

char NaminpicF1[80],NaminpicF2[80],NamoutpicF[80];
char NamseqF[80];

int recgray(Tpic * picture_1, Tpic * picture_2)
{
  int pic_row, pic_col,q_row,q_col;
  int i,j,max,v,pic1=0,pic2,min;
  int i0,iR,j0,jC;
  start=end=NULL;

  //scan I in raster order
  for (pic_row = 0; pic_row <= UR1; pic_row++)
  for (pic_col = 0; pic_col <= UC1; pic_col++)
  {  //NGplus
    max=GRE(picture_2,pic_col,pic_row);
    i0=(pic_row==0)?0:-1; iR=0;
    j0=(pic_col==0)?0:-1; jC=(pic_col==UC1)?0:1;
    for(i=i0;i<=iR;i++)
    for(j=j0;j<=jC;j++)
     if(i==-1||j==-1){v=GRE(picture_2,pic_col+j,pic_row+i);if(v>max)max=v;}
   if ((v=GRE(picture_1,pic_col,pic_row))>max)v=max;
    SetGRE(picture_2,pic_col,pic_row, (Tbrh)v);
  }
  //scan I in anti-raster order
  for (pic_row = UR1; pic_row >= 0; pic_row--)
  for (pic_col = UC1; pic_col >= 0; pic_col--)
  {  //NGminus
    max=0;min=255;
    i0=(pic_row==UR1)?0:1; iR=0;
    j0=(pic_col==UC1)?0:1; jC=(pic_col==0)?0:-1;
    for(i=i0;i>=iR;i--)
    for(j=j0;j>=jC;j--)
     if(i==1||j==1)
     {
      if((v=GRE(picture_2,pic_col+j,pic_row+i))>max)max=v;
      if(v<min){min=v;pic1=GRE(picture_1,pic_col+j,pic_row+i);}
     }
    if((v=GRE(picture_2,pic_col,pic_row))>max)max=v;
    if((v=GRE(picture_1,pic_col,pic_row))>max)v=max;
    SetGRE(picture_2,pic_col,pic_row, (Tbrh)v);
    if(min<max && min<pic1)end=Insert(pic_row,pic_col);
  }
//Propagation step
  while (end!=NULL)
  {
   start=Remove(&pic_row,&pic_col);
   pic2=GRE(picture_2,pic_col,pic_row);
   i0=(pic_row==0)?0:-1; iR=(pic_row==UR1)?0:1;
   j0=(pic_col==0)?0:-1; jC=(pic_col==UC1)?0:1;
     for(i=i0;i<=iR;i++)for(j=j0;j<=jC;j++)//NG
     if(i!=0 || j!=0)
     {
      q_row=pic_row+i;q_col=pic_col+j;
      v=GRE(picture_2,q_col,q_row);
      pic1=GRE(picture_1,q_col,q_row);
      if(v<pic2 && v!=pic1)
      {
       v=(pic2<pic1)?pic2:pic1;
       SetGRE(picture_2,q_col,q_row, (Tbrh)v);
       end=Insert(q_row,q_col);
      }
     }
   }
  return OK;
} /* END of rec1 */


